import { update } from "./objects";
export function createDict() {
    return Object.create(null);
}
export function copyDict(dict) {
    return Object.assign(createDict(), dict);
}
export function hasKey(dict, key) {
    return Object.prototype.hasOwnProperty.call(dict, key.toString());
}
export function mapValues(dict, map) {
    const ret = createDict();
    let changed = false;
    for (let key of Object.keys(dict)) {
        ret[key] = map(dict[key]);
        if (ret[key] !== dict[key])
            changed = true;
    }
    return changed ? ret : dict;
}
export function union(target, source) {
    return update(target, source);
}
export function setKey(dict, key, value) {
    if (dict[key] === value)
        return dict;
    const ret = copyDict(dict);
    ret[key] = value;
    return ret;
}
export function removeKey(dict, ...keys) {
    const ret = copyDict(dict);
    let changed = false;
    for (const key of keys) {
        if (hasKey(ret, key.toString())) {
            changed = true;
            delete ret[key];
        }
    }
    return changed ? ret : dict;
}
export function fromKeys(keys, values) {
    const ret = createDict();
    const isFunction = typeof values === 'function';
    for (const key of keys) {
        ret[key] = isFunction ? values(key) : values;
    }
    return ret;
}
