"use strict";

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.createDict = createDict;
exports.copyDict = copyDict;
exports.hasKey = hasKey;
exports.mapValues = mapValues;
exports.union = union;
exports.setKey = setKey;
exports.removeKey = removeKey;
exports.fromKeys = fromKeys;

var _objects = require("./objects");

function createDict() {
    return Object.create(null);
}
function copyDict(dict) {
    return Object.assign(createDict(), dict);
}
function hasKey(dict, key) {
    return Object.prototype.hasOwnProperty.call(dict, key.toString());
}
function mapValues(dict, map) {
    var ret = createDict();
    var changed = false;
    var _iteratorNormalCompletion = true;
    var _didIteratorError = false;
    var _iteratorError = undefined;

    try {
        for (var _iterator = Object.keys(dict)[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var key = _step.value;

            ret[key] = map(dict[key]);
            if (ret[key] !== dict[key]) changed = true;
        }
    } catch (err) {
        _didIteratorError = true;
        _iteratorError = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion && _iterator.return) {
                _iterator.return();
            }
        } finally {
            if (_didIteratorError) {
                throw _iteratorError;
            }
        }
    }

    return changed ? ret : dict;
}
function union(target, source) {
    return (0, _objects.update)(target, source);
}
function setKey(dict, key, value) {
    if (dict[key] === value) return dict;
    var ret = copyDict(dict);
    ret[key] = value;
    return ret;
}
function removeKey(dict) {
    var ret = copyDict(dict);
    var changed = false;

    for (var _len = arguments.length, keys = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        keys[_key - 1] = arguments[_key];
    }

    var _iteratorNormalCompletion2 = true;
    var _didIteratorError2 = false;
    var _iteratorError2 = undefined;

    try {
        for (var _iterator2 = keys[Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
            var key = _step2.value;

            if (hasKey(ret, key.toString())) {
                changed = true;
                delete ret[key];
            }
        }
    } catch (err) {
        _didIteratorError2 = true;
        _iteratorError2 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion2 && _iterator2.return) {
                _iterator2.return();
            }
        } finally {
            if (_didIteratorError2) {
                throw _iteratorError2;
            }
        }
    }

    return changed ? ret : dict;
}
function fromKeys(keys, values) {
    var ret = createDict();
    var isFunction = typeof values === 'function';
    var _iteratorNormalCompletion3 = true;
    var _didIteratorError3 = false;
    var _iteratorError3 = undefined;

    try {
        for (var _iterator3 = keys[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
            var key = _step3.value;

            ret[key] = isFunction ? values(key) : values;
        }
    } catch (err) {
        _didIteratorError3 = true;
        _iteratorError3 = err;
    } finally {
        try {
            if (!_iteratorNormalCompletion3 && _iterator3.return) {
                _iterator3.return();
            }
        } finally {
            if (_didIteratorError3) {
                throw _iteratorError3;
            }
        }
    }

    return ret;
}